# -*- coding: UTF-8 -*-
#
# Получение сообщений из очереди
#

from artix.common import py2to3utils
from plugins import status
import base64
import logger
import requests
import traceback

requestTimeout = 60
jsonHeader = {'Content-Type': 'application/json', "Accept": "application/json"}
loggerSync = logger.getLogger()
messageTypeKey = 'type'
goodMessageFile = 'lastGoodMessage.txt'


class Message(object):
    def __init__(self, id, content, properties={}):
        self.id = str(id)
        # бинарные данные при передаче через json закодированы с помощью base64
        self.content = base64.b64decode(content)
        self.properties = properties


def saveLastGoodMessageId(messageId):
    try:
        with open(goodMessageFile, 'w') as outfile:
            outfile.write(messageId)
    except Exception as e:
        loggerSync.warn(e)


def getLastGoodMessageId():
    try:
        with open(goodMessageFile, 'r') as infile:
            for line in infile:
                if line and line.strip():
                    return line
    except IOError as e:
        loggerSync.warn(e)
    return None


def getMessage(host, port, endpointId):
    loggerSync.info('Проверяем наличие сообщений в очереди')
    url = 'http://%s:%s/CSrest/rest/v2/queue/findFirst/%s' % (host, port, endpointId)
    with requests.Session() as session:
        response = session.get(url, headers=jsonHeader, timeout=requestTimeout)
        try:
            response.raise_for_status()
            data = response.json()
            return Message(data.get('id'), data.get('body'), data.get('messageProperties', {}))
        except:
            return None


def deleteMessage(host, port, endpointId, messageId):
    loggerSync.info('Удаляем сообщение из очереди')
    url = 'http://%s:%s/CSrest/rest/v2/queue/delete/%s?messageId=%s' % (host, port, endpointId, messageId)
    with requests.Session() as session:
        response = session.delete(url, headers=jsonHeader, timeout=requestTimeout)
        response.raise_for_status()


def main(syncProperties):
    res = status.OK  # результат. Если проверка очереди прошла нормально, то вернется 1
    mess = 'Проверка сообщений закончена'
    try:
        while True:
            message = getMessage(syncProperties.httpHostCS, syncProperties.httpPortCS, syncProperties.cashCode)
            if not message:
                break

            messageId = message.id
            if messageId and messageId == getLastGoodMessageId():
                loggerSync.info('Сообщение с идентификатором %s уже было обработано. Удаляем его.' % messageId)
                deleteMessage(syncProperties.httpHostCS, syncProperties.httpPortCS, syncProperties.cashCode, messageId)
                continue

            loggerSync.info('Получили сообщение')
            messageType = message.properties.get(messageTypeKey, '').lower()
            if not messageType:
                loggerSync.error("Тип сообщения не указан")
                continue
            try:
                plugin = __import__('plugins.%s' % messageType, fromlist='q')
                # выполняем скрипт, соответствующий типу сообщения
                loggerSync.info('Вызываем плагин %s для обработки сообщения' % py2to3utils.ensureStr(messageType))
                try:
                    res, mess = plugin.main(message)
                except Exception as e:
                    mess = str(e)
                    res = status.errorProgramm

                if res != status.OK:
                    return res, mess

                loggerSync.info('Удаляем из очереди принятое сообщение %s' % messageId)
                saveLastGoodMessageId(messageId)
                deleteMessage(syncProperties.httpHostCS, syncProperties.httpPortCS, syncProperties.cashCode, messageId)
            except ImportError as e:
                res = status.errorProgramm
                mess = "Неизвестный тип сообщения. " + str(e)
                loggerSync.error(mess)
            loggerSync.info('')
    except Exception as e:
        res = status.errorProgramm
        mess = str(e)
        loggerSync.error(str(e))
        loggerSync.error(traceback.format_exc())
    finally:
        loggerSync.info('-----------------------')
    return res, mess


if __name__ == '__main__':
    from propertyReader import properties
    main(properties)
